<?php
/*
Plugin Name: SimpleTicker
Plugin URI: http://mbartel.ghulbus.eu/SimpleTicker/
Description: Simple news ticker with multiple input possiblities
Version: 0.8
Author: Michael Bartel
Author URI: http://facebook.com/bartel.michael/
License: MIT/X11
*/

include 'Template.class.php';
$simpleTickerVersion = "0.8";

/*
 * Include wordpress config and database files
 */
@include_once (dirname(dirname(dirname(dirname(__FILE__)))) . DIRECTORY_SEPARATOR. "wp-config.php");
@include_once (dirname(dirname(dirname(dirname(__FILE__)))) . DIRECTORY_SEPARATOR."wp-includes/wp-db.php");

$wp_version = get_bloginfo('version');

/*
 * handle AJAX requests
 */
if ($_GET['action'] == 'getTickerDetails') {
    echo json_encode(getAJAXSimpleTickerDetails($_GET['id']));
    die();
}
if ($_GET['action'] == 'getTickerMessages') {
    echo json_encode(getAJAXSimpleTickerMessages($_GET['id'], $_GET['count'], $_GET['timeout']));
    die();
}

/*
 * If no ticker ID is given by the id GET parameter, try to get the ID with the name parameter
 */
$id_SimpelTicker = isset($_GET['id']) && $_GET['id'] != '' ? $_GET['id'] : getSimpleTickerIDByName($_GET['name']);

/*
 * Handle page name request
 */
if ($_GET['action'] == 'getBlogName') {
    echo get_bloginfo('name');
    die();
}
/*
 * Handle JSON requests
 */
if ($_GET['action'] == 'jsonGetTickerList') {
    echo json_encode(getSimpleTickerList());
    die();
}
if ($_GET['action'] == 'jsonGetTickerMessages') {
    echo json_encode(getSimpleTickerMessages($id_SimpelTicker));
    die();
}
if ($_GET['action'] == 'jsonManageTicker') {
    manageSimpleTicker($id_SimpelTicker, $_GET['data']);
}

/*
 * Handle XML request
 */
if ($_GET['action'] == 'xmlGetTickerList') {
    echo getXMLSimpleTickerList();
    die();
}
if ($_GET['action'] == 'xmlGetTickerMessages') {
    echo getXMLSimpleTickerMessages($id_SimpelTicker);
    die();
}

/*
 * RSS Feed
 */
if ($_GET['action'] == "rssFeed") {
    header("Content-Type: application/rss+xml");
    echo getSimpleTickerRSSFeed($id_SimpelTicker);
    die();
}

/*
 * Plugin installation
 */
register_activation_hook(__FILE__, 'installSimpleTicker');
function installSimpleTicker() {
    global $wpdb;

    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

    $sql = "CREATE TABLE " . $wpdb->prefix . "SimpleTicker (
          `id` INT(11) NOT NULL AUTO_INCREMENT,
          `name` VARCHAR(100) NOT NULL,
          `updateInterval` INT(11) NOT NULL,
          `messageTimeout` INT(11) NOT NULL,
          `messageCount` INT(11) NOT NULL,
          `tickerTimeout` INT(11) NOT NULL,
          `passwd` VARCHAR(250) DEFAULT NULL,
          UNIQUE KEY id (id)
        );";
    
    dbDelta($sql);

    $sql = "CREATE TABLE `svlk_SimpleTickerMsgs` (
          `id` int(11) UNSIGNED NOT NULL AUTO_INCREMENT,
          `id_SimpleTicker` INT(11) DEFAULT NULL,
          `message` VARCHAR(250) DEFAULT NULL,
          `createdOn` DATETIME DEFAULT NULL,
          PRIMARY KEY (`id`)
        );";

    dbDelta($sql);
}

/*
 * add javascript and styles
 */
add_action('init', 'initSimpleTicker');
function initSimpleTicker() {
    $myStyleUrl = plugins_url('simpleticker.css', __FILE__); // Respects SSL, simpleticker.css is relative to the current file
    $myStyleFile = WP_PLUGIN_DIR . '/SimpleTicker/simpleticker.css';
    if (file_exists($myStyleFile)) {
        wp_register_style('simpleTickerStyleSheets', $myStyleUrl);
        wp_enqueue_style( 'simpleTickerStyleSheets');
    }
    wp_enqueue_script("jquery");
    $myJavaScriptUrl = plugins_url('SimpleTicker.js', __FILE__); // Respects SSL, simpleticker.css is relative to the current file
    $myJavaScriptFile = WP_PLUGIN_DIR . '/SimpleTicker/SimpleTicker.js';
    if (file_exists($myJavaScriptFile)) {
        wp_register_script('simpleTickerJavaScripts', $myJavaScriptUrl);
        wp_enqueue_script( 'simpleTickerJavaScripts');
    }
}

/*
 * Uninstall plugin
 */
register_deactivation_hook(__FILE__, 'uninstallSimpleTicker');
function uninstallSimpleTicker() {
    global $wpdb;
    $wpdb->query("DROP TABLE " . $wpdb->prefix ."SimpleTicker");
    $wpdb->query("DROP TABLE " . $wpdb->prefix ."SimpleTickerMsgs");
}

/*
 * provide plugins base URL to JavaScript for AJAX requests
 */
add_action('wp_head', 'simpleTickerBaseURL');
function simpleTickerBaseURL() {
    echo '<script type="text/javascript">var simpleTickerBaseURL = "' . plugins_url('', __FILE__) . '/";</script>';
}

/*
 * create the ticker code itself
 */
add_shortcode('simpleticker', 'printSimpleTicker');
function printSimpleTicker($atts) {
    // we need this to extract the id from [simpleticker id=0]
    extract(shortcode_atts(array('id'=>1), $atts));
    
    return '<div id="SimpleTicker' . $id . '" class="SimpleTicker"><span></span></div>';
}


// WP_PLUGIN_DIR
// plugins_url()

/*
 * Register admin menu page in plugins submenu
 */
add_action('admin_menu', 'showSimpleTickerAdminPage');
function showSimpleTickerAdminPage() {
    add_submenu_page('plugins.php', __('SimpleTicker'), __('SimpleTicker'), 10, 'simpletickeradmin', 'printSimpleTickerAdminPage');
}
function printSimpleTickerAdminPage() {
    global $wpdb;

    /*
     * update the list of tickers
     */
    if (count($_POST['ticker']) > 0) {
        $tickers = $_POST['ticker'];
        foreach ($tickers as $tickerId => $ticker) {
            if ($tickerId == 'new') {
                if ($ticker['name'] != '') {
                    if ($ticker['passwd'] == '') {
                        $query = "INSERT INTO  " . $wpdb->prefix ."SimpleTicker (name, updateInterval, messageTimeout, messageCount, tickerTimeout) " .
                                "VALUES('" . $ticker['name'] . "', '" . $ticker['updateInterval'] . "', '" . $ticker['messageTimeout'] . "', '" .
                                $ticker['messageCount'] . "', '" . $ticker['tickerTimeout'] . "')";
                    } else {
                        $query = "INSERT INTO  " . $wpdb->prefix ."SimpleTicker (name, updateInterval, messageTimeout, messageCount, tickerTimeout, passwd) " .
                                "VALUES('" . $ticker['name'] . "', '" . $ticker['updateInterval'] . "', '" . $ticker['messageTimeout'] . "', '" .
                                $ticker['messageCount'] . "', '" . $ticker['tickerTimeout'] . "', '" . md5($ticker['passwd']) . "')";
                    }
                    $wpdb->query($query);
                }
                unset($tickers[$tickerId]);
                continue;
            }
            if ($ticker['passwd'] == '') {
                $wpdb->query("UPDATE " . $wpdb->prefix ."SimpleTicker SET name='" . $ticker['name'] . "', updateInterval='" . $ticker['updateInterval'] .
                    "', messageTimeout='" . $ticker['messageTimeout'] . "', messageCount='" . $ticker['messageCount'] . "', tickerTimeout='" . $ticker['tickerTimeout'] .
                    "' WHERE id='$tickerId'");
            } else {
                $wpdb->query("UPDATE " . $wpdb->prefix ."SimpleTicker SET name='" . $ticker['name'] . "', updateInterval='" . $ticker['updateInterval'] .
                    "', messageTimeout='" . $ticker['messageTimeout'] . "', messageCount='" . $ticker['messageCount'] . "', tickerTimeout='" . $ticker['tickerTimeout'] .
                    "', passwd='" . md5($ticker['passwd']) . "' WHERE id='$tickerId'");
            }

            if ($ticker['delete'] == "1") {
                $wpdb->query("DELETE FROM " . $wpdb->prefix ."SimpleTicker WHERE id='$tickerId'");
                $wpdb->query("DELETE FROM " . $wpdb->prefix ."SimpleTickerMsgs WHERE id_SimpleTicker='$tickerId'");
            }
        }
    }
    
    /*
     * update the list of messages
     */
    if (count($_POST['msg']) > 0) {
        $msgs = $_POST['msg'];
        foreach ($msgs as $msgId => $msg) {
            if ($msgId == 'new') {
                if ($msg['message'] != '') {
                    $query = "INSERT INTO  " . $wpdb->prefix ."SimpleTickerMsgs (id_SimpleTicker, message, createdOn) " .
                                "VALUES('" . $msg['ticker'] . "', '" . $msg['message'] . "', NOW())";
                    $wpdb->query($query);
                }
            } elseif ($msg['delete'] == "1") {
                $wpdb->query("DELETE FROM " . $wpdb->prefix ."SimpleTickerMsgs WHERE id='$msgId'");
            }
        }
    }

    /*
     * print the list of tickers
     */
    $adminPage = new Template('adminpage.tpl');
    $adminPage->assign('pluginpath', plugins_url('', __FILE__));

    $tickers = $wpdb->get_results("SELECT * FROM " . $wpdb->prefix . "SimpleTicker");
    $tickerList = array();
    $tickerListArr = array();
    foreach ($tickers as $ticker) {
        $tickerList[$ticker->id] = $ticker->name;
        $tmp = array();
        $tmp['id'] = $ticker->id;
        $tmp['name'] = $ticker->name;
        $tmp['updateInterval'] = $ticker->updateInterval;
        $tmp['messageTimeout'] = $ticker->messageTimeout;
        $tmp['messageCount'] = $ticker->messageCount;
        $tmp['tickerTimeout'] = $ticker->tickerTimeout;
        $tickerListArr[] = $tmp;
    }
    $adminPage->assignBlock('tickerlist', $tickerListArr);


    $messages = $wpdb->get_results("SELECT " . $wpdb->prefix . "SimpleTickerMsgs.id AS tickerId, id_SimpleTicker, name, message, createdOn FROM " .
                                        $wpdb->prefix . "SimpleTickerMsgs JOIN " .
                                        $wpdb->prefix . "SimpleTicker ON " . $wpdb->prefix . "SimpleTickerMsgs.id_SimpleTicker = " .
                                        $wpdb->prefix . "SimpleTicker.id ORDER BY createdOn DESC LIMIT 50");
    $messagesArr = array();
    foreach ($messages as $message) {
        $tmp = array();
        $tmp['tickerId'] = $message->tickerId;
        $tmp['name'] = $message->name;
        $tmp['message'] = $message->message;
        $tmp['createdOn'] = $message->createdOn;
        $messagesArr[] = $tmp;
    }
    $adminPage->assignBlock('messagelist', $messagesArr);

    $code = '';
    foreach ($tickerList as $key => $value) {
        $code .= "                <option value=\"$key\">$value</option>";
    }
    $adminPage->assign('tickeridlist', $code);
    

    $adminPage->display();
}

/**
 * Returns the decrypted string
 * @param <string> $str
 * @return <string>
 */
function decryptSimpleTickerMessage($str) {
    return $str; //TODO implement decryption
}

/**
 * Returns the ticker ID by it's name
 * @param <type> $name 
 */
function getSimpleTickerIDByName($name) {
    global $wpdb;
    $query = "SELECT id FROM " . $wpdb->prefix . "SimpleTicker WHERE name='$name'";
    $row = $wpdb->get_Row($query);
    return $row->id;
}


/**
 * Return detailed information about a specific ticker (defined by it's ID)
 * @global <type> $wpdb
 * @param <type> $id_SimpleTicker
 * @return <type>
 */
function getAJAXSimpleTickerDetails($id_SimpleTicker) {
    global $wpdb;
    $query = "SELECT name, updateInterval, messageTimeout, messageCount, tickerTimeout FROM " . $wpdb->prefix . "SimpleTicker WHERE id='" . mysql_escape_string($id_SimpleTicker) . "'";
    $queryResult = $wpdb->get_row($query);

    $result = array();
    $result['updateInterval'] = $queryResult->updateInterval * 60;
    $result['messageTimeout'] = $queryResult->messageTimeout;
    $result['messageCount'] = $queryResult->messageCount;
    $result['tickerTimeout'] = $queryResult->tickerTimeout * 60;
    $result['name'] = $queryResult->name;

    return $result;
}

/**
 * Return all currently 'active' messages (messages, where the timeout hasn't been exceeded) for a ticker with a specific ID
 * @global <type> $wpdb
 * @param <type> $id_SimpleTicker
 * @param <type> $messageCount
 * @param <type> $messageTimeout
 */
function getAJAXSimpleTickerMessages($id_SimpleTicker, $messageCount, $messageTimeout) {
    global $wpdb;
    $query = "SELECT message FROM " . $wpdb->prefix . "SimpleTickerMsgs WHERE " .
            "id_SimpleTicker='" . mysql_escape_string($id_SimpleTicker) . "' AND createdOn > '" . date("Y-m-d h:i:s", time() - mysql_escape_string($messageTimeout) * 60) . "' " .
            "ORDER BY createdOn DESC LIMIT " . mysql_escape_string($messageCount);
    $messages = $wpdb->get_results($query);

    $result = array();
    foreach ($messages as $message) {
        $result[] = $message->message;
    }

    return $result;
}

/**
 * Returns a list of all ticker (ID and name)
 * @global  $wpdb
 * @return <type>
 */
function getSimpleTickerList() {
    global $wpdb;
    $tickers = $wpdb->get_results("SELECT * FROM " . $wpdb->prefix . "SimpleTicker");
    $tickerList = array();
    foreach ($tickers as $ticker) {
        $tickerList[] = array('id' => $ticker->id, 'name' => $ticker->name);
    }
    return $tickerList;
}

/**
 * Returns a list with the 50 latest messages for the ticker with the given ticker ID
 * @global $wpdb $wpdb
 * @param <type> $id_SimpleTicker
 * @return <type>
 */
function getSimpleTickerMessages($id_SimpleTicker) {
    global $wpdb;
    $messages = $wpdb->get_results("SELECT id, message, createdOn FROM " .
                                        $wpdb->prefix . "SimpleTickerMsgs WHERE id_SimpleTicker = '" . mysql_escape_string($id_SimpleTicker) .
                                        "' ORDER BY createdOn DESC LIMIT 50");
    $tickerMessages = array();
    foreach ($messages as $message) {
        $uid = substr(str_pad(str_replace(array(' ', ':', '-'), array('', '', ''), $id_SimpleTicker . $message->id . $message->createdOn. $message->message), 32, 'A'), 0, 32);
        $tickerMessages[] = array('id' => $message->id, 'message' => $message->message, 'createdOn' => $message->createdOn, 'uid' => $uid);
    }

    return $tickerMessages;
}

/**
 * Manage tickers with JSON request and encrypted BASE64 data
 * @global $wpdb;
 * @param <type> $id_SimpleTicker
 * @param <type> $data 
 */
function manageSimpleTicker($id_SimpleTicker, $data) {
    global $wpdb;
    $id_SimpleTicker = mysql_escape_string($id_SimpleTicker);

    /*
     * Check if the password encrypted with the token matches the password for this ticker stored in the database.
     */
    $decryptedData = json_decode(decryptSimpleTickerMessage(base64_decode($data)), true);
    if ($decryptedData == null) {
        die("NODATACONTENT");
    }
    $tickerData = $wpdb->get_row("SELECT passwd FROM " . $wpdb->prefix . "SimpleTicker WHERE id='" . mysql_escape_string($id_SimpleTicker) . "'");
    if (md5($decryptedData['passwd']) == $tickerData->passwd) {
        /*
         * Add new messsage
         */
        if ($decryptedData['action'] == 'addMessage') {
            $message = $decryptedData['message'];
            $wpdb->query("INSERT INTO " . $wpdb->prefix . "SimpleTickerMsgs (id_SimpleTicker, message, createdOn) VALUES ('$id_SimpleTicker', '$message', NOW())");
            die("SUCCESS");
        }
        /*
         * Delete existing message
         */
        if ($decryptedData['action'] == 'removeMessage') {
            $id = $decryptedData['id'];
            $wpdb->query("DELETE FROM " . $wpdb->prefix . "SimpleTickerMsgs WHERE id='$id'");
            die("SUCCESS");
        }
        die("NOACTIONDEFINED");
    } else {
        die("WRONGPASSWORD");
    }
}

/**
 * Returns an XML string representation of getSimpleTickerList
 * @return string
 */
function getXMLSimpleTickerList() {
    $tickers = getTickerList();

    $tickerStr = "<tickerlist>\n";
    foreach ($tickers as $ticker) {
        $tickerStr .= "\t<ticker>\n\t\t<id>" . $ticker['id'] . "</id>\n\t\t<name>" . $ticker['name'] . "</name>\n\t</ticker>\n";
    }
    $tickerStr .= "</tickerlist>";

    return $tickerStr;
}

/**
 * Returns an XML string representation of getSimpleTickerMessages
 * @param <type> $id_SimpleTicker
 * @return string
 */
function getXMLSimpleTickerMessages($id_SimpleTicker) {
    $messages = getSimpleTickerMessages($id_SimpleTicker);

    $tickerMessages = "<tickermessages>\n";
    foreach ($messages as $message) {
        $tickerMessages .= "\t<message>\n\t\t<id>" . $message['id'] . "</id>\n\t\t<message>" . $message['message'] .
                            "</message>\n\t\t<createdOn>" . $message['createdOn'] . "</createdOn>\n\t</message>\n";
    }
    $tickerMessages .= "</tickermessages>";

    return $tickerMessages;
}

/**
 * Create string content for the RSS Feed
 * @param <type> $id_SimpleTicker
 */
function getSimpleTickerRSSFeed($id_SimpleTicker) {
    $tickerInfo = getAJAXSimpleTickerDetails($id_SimpleTicker);
    $messages = getSimpleTickerMessages($id_SimpleTicker);

    $feed = new Template('rssFeed.tpl');
    $feed->assign('name', $tickerInfo['name']);
    $feed->assignBlock('messages', $messages);

    $lastMsg = $messages[0];

    $feed->assign('lastCreatedOn', $lastMsg['createdOn']);
    $feed->assign('currentTimestamp', date("Y-m-d h:i:s", time()));

    return $feed->fetch();
}

?>
